#include "AneuMeshLoader.h"
#include <fstream>
#include <algorithm>

void AneuMeshLoader::loadMesh(const std::string& p_fileName) {
    std::ifstream file(p_fileName);
    if (!file) {
        throw FileNotFound();
    }
    else std::cout << "File " << p_fileName << " was opened successfully." << std::endl;


    // 
    int count;
    file >> count;
    file.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
    Nodes.reserve(count);
    for (int i = 0; i < count; ++i) {
        Node temp;
        file >> temp.m_coordinates[0] >> temp.m_coordinates[1] >> temp.m_coordinates[2];
        temp.m_vertexDot = false;
        Nodes.insert({ m_nodeId.generateId(), std::move(temp) });
    }


    // 
    file >> count;
    file.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
    FiniteElements.reserve(count);
    for (int i = 0; i < count; ++i) {
        int geometryAreaID;
        file >> geometryAreaID;
        std::vector<int> idLst(Tetrahedron::m_size);
        for (int j = 0; j < Tetrahedron::m_size; ++j) {
            file >> idLst[j];
        }
        Tetrahedron temp(geometryAreaID, idLst);
        FiniteElements.insert({ m_finiteElementId.generateId(), std::move(temp) });
    }


    //  
    file >> count;
    file.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
    BoundaryElements.reserve(count);
    for (int i = 0; i < count; ++i) {
        int geometryAreaID;
        file >> geometryAreaID;
        std::vector<int> idLst(Triangle::m_size);
        for (int j = 0; j < Triangle::m_size; ++j) {
            file >> idLst[j];
        }
        Triangle temp(geometryAreaID, idLst);
        BoundaryElements.insert({ m_boundaryElementId.generateId(), std::move(temp) });
    }
}