#include "Components.h"
#include <iomanip>


Node::Node(const std::array<double, 3>& p_coordinates, bool p_VertexDot) :
    m_coordinates(p_coordinates), m_vertexDot(p_VertexDot) {}



//  
std::ostream& operator<<(std::ostream& out, const Node& p_node) {
    out << "Node coordinate: (" << p_node.m_coordinates[0] << ";" << p_node.m_coordinates[1] << ";" << p_node.m_coordinates[2] << ")\t";
    if (p_node.m_vertexDot)
        out << "Node is a vertex\n";
    else
        out << "Node is internal\n";
    out << std::endl;
    return out;
}


std::ostream& operator<<(std::ostream& out, const std::unordered_map<int, Node>& p_node) {
    int i = 1;
    for (const auto& node : p_node) {
        out << "Node ID: " << node.first << " " << node.second;
    }
    return out;
};


FiniteElement::FiniteElement(int p_GeometryAreaID, const std::vector<int>& p_NodeIDs) :
    m_GeometryAreaID(p_GeometryAreaID), m_NodeIDs(p_NodeIDs) {}


//  
std::ostream& operator<<(std::ostream& out, const Tetrahedron& p_Tetrahedron) {
    out << "Geometry Area ID: " << p_Tetrahedron.m_GeometryAreaID << "\t";
    out << "Nodes of Finite Element: ";
    for (auto& node : p_Tetrahedron.m_NodeIDs) {
        out << node << " ";
    }
    out << "\n" << std::endl;
    return out;
}


std::ostream& operator<<(std::ostream& out, const std::unordered_map<int, Tetrahedron>& p_Tetrahedrons) {
    for (const auto& elem : p_Tetrahedrons)
        out << "Finite Element ID: " << elem.first << " " << elem.second;
    return out;
}


std::ostream& operator<<(std::ostream& out, const Triangle& p_Triangle) {
    out << "Geometry Area ID: " << p_Triangle.m_GeometryAreaID << "\t";
    out << "Nodes of Boundary Element: ";
    for (auto& node : p_Triangle.m_NodeIDs) {
        out << node << " ";
    }
    out << "\n" << std::endl;
    return out;
}


std::ostream& operator<<(std::ostream& out, const std::unordered_map<int, Triangle>& p_Triangle) {
    for (const auto& elem : p_Triangle)
        out << "Boundary Element ID: " << elem.first << " " << elem.second;
    return out;
}



Edge::Edge(int p_firstNodeId, int p_secNodeId)
    : m_edgeNodes({ p_firstNodeId, p_secNodeId }) {}


Edge& Edge::operator=(const Edge& p_edge) {
    if (this != &p_edge) {
        this->m_edgeNodes = p_edge.m_edgeNodes;
        this->m_midNode = p_edge.m_midNode;
    }
    return *this;
}


bool Edge::operator==(const Edge& p_edge) const {
    return ((m_edgeNodes.first == p_edge.m_edgeNodes.first) &&
        (m_edgeNodes.second == p_edge.m_edgeNodes.second)) || ((m_edgeNodes.first == p_edge.m_edgeNodes.second) &&
            (m_edgeNodes.second == p_edge.m_edgeNodes.first));
}