#pragma once
#include <iostream>
#include <vector>
#include <array>
#include <unordered_map>

class IdGenerator {
public:
	IdGenerator() : currentId(1) {}
	int generateId() {
		return currentId++;
	}
private:
	int currentId;
};


struct Node {
	static constexpr int dim = 3;
	std::array<double, dim> m_coordinates{};
	bool m_vertexDot;

	Node() = default;
	Node(const std::array<double, dim>&, bool);

	friend std::ostream& operator<<(std::ostream&, const Node&);
	friend std::ostream& operator<<(std::ostream&, const std::unordered_map<int, Node>&);
};


struct FiniteElement {
	int m_GeometryAreaID;
	std::vector<int> m_NodeIDs;
	FiniteElement(int p_GeometryAreaID, const std::vector<int>& p_NodeIDs);
};


class Tetrahedron : public FiniteElement {
public:
	static constexpr int m_size = 4;
	Tetrahedron(int p_geomId, const std::vector<int>& p_nodes) : FiniteElement(p_geomId, p_nodes) {};
	friend std::ostream& operator<<(std::ostream&, const Tetrahedron&);
	friend std::ostream& operator<<(std::ostream&, const std::unordered_map<int, Tetrahedron>&);
};


class Triangle : public FiniteElement {
public:
	static constexpr int m_size = 3;
	Triangle(int p_geomId, const std::vector<int>& p_nodes) : FiniteElement(p_geomId, p_nodes) {};
	friend std::ostream& operator<<(std::ostream&, const Triangle&);
	friend std::ostream& operator<<(std::ostream&, const std::unordered_map<int, Triangle>&);
};


struct Edge {
	std::pair<int, int> m_edgeNodes;
	int m_midNode = -1;
	Edge() = default;
	Edge(int, int);
	Edge& operator=(const Edge&);
	bool operator==(const Edge&) const;
};
