#pragma once
#include <iostream>
#include <stdlib.h>
#include <vector>
#include <set>
#include <array>
#include "Components.h"
#include "unordered_set"
class MeshLoader {
private:
	Node getMiddleNode(const Edge&);
protected:
	IdGenerator m_nodeId;
	IdGenerator m_finiteElementId;
	IdGenerator m_boundaryElementId;
	std::unordered_map<int, Node> Nodes;
	std::unordered_map<int, Tetrahedron> FiniteElements;
	std::unordered_map<int, Triangle> BoundaryElements;
public:
	virtual void loadMesh(const std::string&) = 0;
	virtual ~MeshLoader() = default;

	const std::unordered_map<int, Node>& getNodes();
	const std::unordered_map<int, Tetrahedron>& getFiniteElements();
	const std::unordered_map<int, Triangle>& getBoundaryElements();

	std::unordered_set<int> getFiniteElementIDByNodesID(int, int, int);
	std::unordered_set<int> getFiniteElementIDByEdge(int, int);
	std::unordered_set<int> getNodesIDByBorder(int);
	std::unordered_set<int> getFiniteElementIdBySurface(int);
	std::unordered_set<int> getBoundElementIDByBorder(int);

	std::vector<Tetrahedron> getFiniteElementByNodesID(int, int, int);
	std::vector<Tetrahedron> getFiniteElementByEdge(int, int);
	std::vector<Node> getNodesByBorder(int);
	std::vector<Tetrahedron> getFiniteElementBySurface(int);
	std::vector<Triangle> getBoundElementByBorder(int);

	static void printNodes(const std::unordered_map<int, Node>&);
	static void printFiniteElements(const std::unordered_map<int, Tetrahedron>&);
	static void printBoundaryElements(const std::unordered_map<int, Triangle>&);

	void insertNodeMid();

	std::unordered_map<int, std::set<int>> createNeighborsMap();
};

