#include <algorithm>
#include <fstream>
#include <valarray>
#include <iostream>
#include <algorithm>
#include <string>
#include <ranges>
#include <array>
#include "MeshLoader.h"
#include "AneuMeshLoader.h"
using namespace std;

int main(int argc, char* argv[]) {
	std::string filepath = "./cube.aneu";
	if (argc != 2)
		std::cout << "Wrong number of arguments, using standard file PATH\n" << std::endl;
	else {
		std::cout << "Set file path to " << argv[1] << "\n" << std::endl;
		filepath = argv[1];
	};


	MeshLoader* l = new AneuMeshLoader();
	try {
		l->loadMesh(filepath);
	}
	catch (std::exception& er) {
		std::cerr << er.what() << std::endl;
		delete l;
		return 0;
	}


	std::cout << "Nodes from file: \n" << std::endl;
	l->printNodes(l->getNodes());


	std::cout << "Finite Elements from file: \n" << std::endl;
	l->printFiniteElements(l->getFiniteElements());


	std::cout << "Bouundary Finite Elements from file: \n" << std::endl;
	l->printBoundaryElements(l->getBoundaryElements());


	std::cout << "Neighbours of Nodes: \n" << std::endl;
	l->createNeighborsMap();


	std::cout << "ID Finite elements by nodes (4,1,2)" << std::endl;
	auto finelem_1 = l->getFiniteElementIDByNodesID(4, 1, 2);
	for (int elem : finelem_1) {
		std::cout << elem << " ";
	}
	std::cout << std::endl << "-----------------------------------" << std::endl;


	std::cout << "ID Finite elements by Edge (4,7)" << std::endl;
	auto finelem_2 = l->getFiniteElementIDByEdge(4, 7);
	for (int elem : finelem_2) {
		std::cout << elem << " ";
	}
	std::cout << std::endl << "-----------------------------------" << std::endl;


	std::cout << "ID of Nodes by Geometry Area ID of Bounder element (3)" << std::endl;
	auto finnode = l->getNodesIDByBorder(3);
	for (int node : finnode) {
		std::cout << node << " ";
	}
	std::cout << std::endl << "-----------------------------------" << std::endl;


	std::cout << "ID Finite elements by surface ID (1)" << std::endl;
	auto finelem_3 = l->getFiniteElementIdBySurface(1);
	for (int elem : finelem_3) {
		std::cout << elem << " ";
	}
	std::cout << std::endl << "-----------------------------------" << std::endl;


	std::cout << "Boundary element ID by boundary ID (2)" << std::endl;
	auto finelem_4 = l->getBoundElementIDByBorder(2);
	for (int elem : finelem_4) {
		std::cout << elem << " ";
	}
	std::cout << std::endl << "-----------------------------------" << std::endl;


	std::cout << "inserting middle Elements: \n" << std::endl;
	l->insertNodeMid();


	std::cout << "Nodes after inserting middle Element: \n" << std::endl;
	l->printNodes(l->getNodes());


	std::cout << "Neighbours of new Nodes: \n" << std::endl;
	l->createNeighborsMap();


	delete l;
	return 0;
}